@@ -1,13 +1,34 @@
 from functools import partial, wraps
 from typing import Callable
 
+_ON = True
+
+
+def Toggle():
+    global _ON
+    _ON = not _ON
+
+
+def Off():
+    global _ON
+    _ON = False
+
+
+def On():
+    global _ON
+    _ON = True
+
 
 def call_function(target: Callable, arguments: dict, result=None):
-    # only pass the needed arguments to the function
-    arguments = {key: value for key, value in arguments.items() if key in target.__code__.co_varnames}
-    if result is not None and "_result" in target.__code__.co_varnames:
+    # First Try passing all arguments
+    if result is not None:
         arguments["_result"] = result
-    return target(**arguments)
+    try:
+        return target(**arguments)
+    except TypeError:
+        # Then Try passing only the arguments that are defined in the function
+        arguments = {k: v for k, v in arguments.items() if k in target.__code__.co_varnames}
+        return target(**arguments)
 
 
 class Wrapper:
@@ -22,17 +43,20 @@ class Wrapper:
 
         kwargs.update(zip(var_names, args))
 
-        for before_function in self._before:
-            updates = call_function(before_function, kwargs)
-            if updates:
-                kwargs.update(updates)
+        if _ON:
+            for before_function in self._before:
+                updates = call_function(before_function, kwargs)
+                if updates:
+                    kwargs.update(updates)
 
-        result = call_function(self.function, kwargs)
+            result = call_function(self.function, kwargs)
 
-        for after_function in self._after:
-            result = call_function(after_function, kwargs, result=result)
+            for after_function in self._after:
+                result = call_function(after_function, kwargs, result=result)
 
-        return result
+            return result
+        else:
+            return call_function(self.function, kwargs)
 
     def remove_hook(self, function):
         if function in self._before:
@@ -62,9 +86,6 @@ class Wrapper:
 
 
 def Plugin(function=None):
-    if function is None:
-        return partial(Plugin, function)
-
     # whether function is defined inside class
     if function.__name__ != function.__qualname__:
         wrapper = Wrapper(function, in_class=True)
@@ -75,7 +96,21 @@ def Plugin(function=None):
     def inner(*args, **kwargs):
         return wrapper(*args, **kwargs)
 
+    inner.instance = wrapper
     inner.before = wrapper.before
     inner.after = wrapper.after
 
     return inner
+
+
+# Get Plugin Workflow
+def workflow(function) -> list:
+    try:
+        hooks = function.instance._before + [function] + function.instance._after
+        return hooks
+    except Exception:
+        return [function]
+
+
+def print_workflow(function):
+    print([f.__name__ for f in workflow(function)])
